#!/usr/bin/perl -w
#
# This script handles registered nicks on the Hispano IRC network
#
# Author: ryu@gpul.org
#
# It's designed for X-Chat
#
# This script is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

# This script is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You can read the GNU General Public License at http://www.gnu.org

# * IMPORTANTE *
# Antes de empezar a usar el script, usa el comando /register para que se
# guarden tus password en la base de datos, o bien escribelos en el archivo
# $REGISTER_passwd (ver mas abajo) con el formato <nick>:<password>
#
# Comandos soportados:
# /identify : reenva tu password a NiCK
# /ghost : recupera tu nick si lo tiene otra persona o te has caido y sigue
#          el nick antiguo en IRC.
# /info  : da informacin sobre un nick.


# Esta es la mask de nick, no creo que cambie...
my $NiCK_mask="-\@-";

# Este es el archivo donde se almacenarn los password
my $REGISTER_passwd="$ENV{HOME}/.xchat/register_passwd";

my $ghost_nick=0; # Para cambiarse de nick despues de la confirmacion de un GHOST
my @waiting_op=(); # Para eliminar el mensaje de chan de error cuando sea
		   # register quien le pide op

###########################################################################
## A partir de aqui, mejor no toques sin saber con seguridad lo que haces ##
###########################################################################

my $VERSION="0.3.4";

IRC::add_message_handler("PRIVMSG", "register_privmsg_handler");
IRC::add_message_handler("474", "register_baned_handler");

## Commands ##
my @commands=(
	"register",
	"identify",
	"ghost",
	"info"
	);

my $commands_msg="";

for(@commands){
  IRC::add_command_handler("$_","$_\_handler");
  $commands_msg="$commands_msg /$_";
}

IRC::print "\0030,11 \0030,10 \0030,12 \0030,2 \003 Cargando script \00312\002REGISTER $VERSION\003\n";
IRC::print "\0030,11 \0030,10 \0030,12 \0030,2 \003 Comandos:\00312\002$commands_msg \003\n";

sub get_password
{
    my $nick=lc(shift(@_));
    my $password="";
    open(FILE,"<$REGISTER_passwd");
    my @passwd_list=<FILE>;
    close(FILE);
    foreach my $file_line (@passwd_list)
    {
        $file_line =~ /(.*):(.*)/;
        my $line_nick=$1;
        my $line_pass=$2;
        if ( $line_nick =~ /$nick/ ) {$password=$line_pass;}
    }
    
    return $password;
}

sub change_password
{
    my $nick=lc(shift(@_));
    my $new_pass=shift(@_);
    open(FILE,"<$REGISTER_passwd");
    my @passwd_list=<FILE>;
    close(FILE);

    open(FILE,">$REGISTER_passwd");
    foreach my $file_line (@passwd_list)
    {
        $file_line =~ /(.*):(.*)/;
        my $line_nick=$1;
        my $line_pass=$2;
        if ( $line_nick =~ /$nick/ )
        {
            print FILE "$nick:$new_pass\n";
        }
        else
        {
            print FILE "$line_nick:$line_pass\n";
        }
    }
    close(FILE);
}

sub register_privmsg_handler
{
    # Leemos la lnea con el mensaje proviniente del servidor
    my $line = shift(@_);

    # Dividimos la lnea en partes para quedarnos con el nick, la mask
    # y el mensaje 
    $line =~ /:([^!]*)!([^\@]*\@[^ ]*) PRIVMSG [^:]*:(.*)/;
    my $su_nick= $1;
    my $mask = $2;
    my $msg  = $3;

    if (not ($su_nick =~ /^NiCK$/)) {return 0;}

    # Comprobamos que NiCK nos manda el mensaje tpico que nos pide el
    # password
    if ($msg =~ /^.?.?.?Este nick est.*/ || $msg =~ /^.?.?.?Este nick perte.*/)
    {
        # Comprobamos que la mask es la de NiCK, y no es ningn gracioso
        # intentando pillar passwords
        if ($mask =~ /$NiCK_mask/)
        {
            # Buscamos el password en la base de datos
            my $nick=IRC::get_info(1); # 1 es tu nick, obviamente
            my $password=get_password($nick);
            if (not $password)
            {
                IRC::print("\0034REGISTER WARNING\003: NiCK ha requerido tu password, pero todava no lo tengo en mi base de datos, puedes aadirlo asi: /register <password>.\n");
                return 0;
            }
            # Enviamos a NiCK nuestro password
            IRC::command("/raw PRIVMSG NiCK :identify $password");
            # Devolvemos 1 para que xchat no ejecute la accin por defecto y
            # no saque el mensaje en una ventana.
            return 1;
        }
        else
        {
            IRC::print "\0034REGISTER WARNING\003:Alguien se puso NiCK de nick e intenta obtener tu password!\n";
        }
    }
    elsif ($msg =~ /^.?.?.?Contrasea aceptada.*/)
    {
	IRC::print "\0034**\003 REGISTER: Identificado correctamente\n";
        giveme_massive_op();
	return 1;
    }
    elsif ($msg =~ /.*Si es el tuyo.*/)
    {
        return 1;
    }
    elsif ($msg =~ /^.?.?.?Si no cambias tu nick.*/)
    {
	return 1;
    }
    elsif ($msg =~ /^.?.?.?Invisible con su nick ha.*/)
    {
	IRC::print "\0034**\003 REGISTER: Ghost funcion correctamente\n";
	if ($ghost_nick) {
	  IRC::command("/nick $ghost_nick");
	  $ghost_nick=0;
	}
	return 1;
    }

   return 0;
}

# /register command handler
sub register_handler
{
    my $passwd = shift(@_);
    my $nick=lc(IRC::get_info(1)); # 1 es tu nick, obviamente

    push @ignore_list, $luser;
#    IRC::command("/msg NiCK identify $passwd");
    $test=get_password($nick);
    if ($test)
    {
        change_password($nick,$passwd);
        IRC::print "\0034**\003 REGISTER: El password para tu nick ha sido \002cambiado\002 en la base de datos \0034**\003 \n";
    }
    else
    {
        open(FILE,">>$REGISTER_passwd");
        print FILE  "$nick\:$passwd\n";
        close(FILE);
        IRC::print "\0034**\003 REGISTER: El password para tu nick ha sido almacenado en la base de datos \0034**\003 \n";
    }
    return 1;
}

sub identify_handler
{
    # Buscamos el password en la base de datos
    my $nick=IRC::get_info(1); # 1 es tu nick, obviamente
    my $password=get_password($nick);
    if (not $password)
    {
      IRC::print("\0034REGISTER WARNING\003: todava no tengo tu password en mi base de datos, puedes aadirlo asi: /register <password>.\n");
      return 0;
    }
    # Enviamos a NiCK nuestro password
    IRC::command("/raw PRIVMSG NiCK :identify $password");
    return 1;
}

sub ghost_handler
{
    my $nick=shift(@_); # leemos el nick en cuestion
    if (not $nick) 
    {
        IRC::print("Usage: /ghost <nick>\n");
        return 1;
    }
    # Buscamos el password en la base de datos
    my $password=get_password($nick);
    if (not $password)
    {
      IRC::print("\0034REGISTER WARNING\003: todava no tengo tu password en mi base de datos, puedes aadirlo poniendote el nick y luego haciendo: /register <password>.\n");
      return 1;
    }
    # Enviamos a NiCK un GHOST con nuestro password
    IRC::command("/raw PRIVMSG NiCK :ghost $nick $password");
    $ghost_nick=$nick;
    return 1;
}

sub info_handler
{
    my $nick=shift(@_); # Leemos el nick del que obtener info
    if (not $nick)
    {
        IRC::print("Usage: /info <nick>\n");
        return 1;
    }
    IRC::command("/raw PRIVMSG NiCK :info $nick");
    return 1;
}

sub giveme_op
{
    my $nick=IRC::get_info(1); # 1 es tu nick, obviamente
    my $channel=shift(@_);
    IRC::command("/raw PRIVMSG CHaN :op $channel $nick");
    push @waiting_op, $channel;
}

sub giveme_massive_op
{
    my @channels=IRC::channel_list();
    my $chan=0;
    my $serv=0;
    my $nick=0;
    for (@channels){
      if (not $chan) {
        $chan=$_;
      }elsif (not $serv){
        $serv=$_;
      }else{
        $nick=$_;
        if ($chan=~/^#/) { # Comprobamos que es un canal, y no otra cosa
          giveme_op($chan);
        }
        $chan=0;$nick=0;$serv=0;
      }
    }
}

sub register_baned_handler
{
    my $line = shift(@_);
    $line =~ /([^ ]*) ([^ ]*) ([^ ]*) ([^ ]*)/;
    my $chan= $4;
    
    #le pedimos a chan que nos quite el ban
    IRC::command("/raw PRIVMSG CHAN :unban $chan");
}                                                 
